
# q-expansion of newform 210.2.i.d, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 210
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[71, [1, 0]], [127, [1, 0]], [31, [0, -1]]]
aps_data = [[0, 1], [1, -1], [0, 1], [1, 2], [1, -1], [1, 0], [0, 0], [0, 3], [0, -7], [-8, 0], [2, -2], [0, -11], [-11, 0], [8, 0], [0, 5], [11, -11], [-4, 4], [0, 0], [0, 0], [-6, 0], [6, -6], [0, 8], [8, 0], [0, 10], [-16, 0], [0, 0], [0, 16], [0, 10], [-6, 6], [6, 0], [-17, 0], [0, 5], [18, -18], [20, 0], [0, 0], [-6, 6], [7, -7], [0, -16], [-3, 0], [0, 15], [19, -19], [-24, 0], [0, 6], [-22, 22], [-1, 0], [24, -24], [5, 0], [12, 0], [-8, 8], [0, -14], [0, -18], [-18, 0], [-7, 7], [13, 0], [0, 20], [-24, 24], [-20, 20], [0, -32], [22, -22], [-1, 0], [14, -14], [27, 0], [20, 0], [-12, 12], [0, 12], [0, 18], [0, 13], [-12, 0], [-14, 14], [12, 0], [-24, 24], [0, -4], [-25, 25], [0, -6], [-19, 0], [0, -35], [-10, 10], [0, -14], [0, 5], [-2, 2], [-5, 0], [22, 0], [0, 0], [-32, 0], [0, -32], [0, -16], [11, 0], [0, 18], [-12, 0], [-13, 0], [0, 0], [-22, 22], [16, -16], [20, 0], [0, 32], [36, 0], [0, 34], [33, -33], [0, -2], [0, 10], [16, 0], [33, -33], [38, -38], [0, 9], [32, -32], [14, -14], [-18, 0], [0, -16], [-2, 2], [-26, 0], [0, 37], [41, -41], [28, 0], [-29, 29], [-26, 0], [3, -3], [-10, 0], [17, -17], [0, -7], [36, 0], [24, -24], [28, 0], [0, 13], [-4, 4], [0, 12], [-18, 0], [0, -44], [0, -2], [11, 0], [0, -11], [19, -19], [49, 0], [0, -26], [2, 0], [0, -27], [-5, 0], [-33, 33], [-22, 22], [-18, 0], [39, -39], [9, 0], [0, -42], [24, -24], [-10, 0], [8, -8], [-40, 0], [-41, 0], [14, -14], [0, -12], [0, 53], [0, -17], [-53, 0], [-42, 0], [0, -44], [10, -10], [-2, 0], [0, 36], [0, 21], [12, 0], [-54, 54], [0, -22], [-20, 0], [-20, 0], [0, 45], [-54, 54], [-49, 49], [-22, 22], [-14, 14]]
